﻿' *****************************************************************************
' IIS Tweak
' A simple utility that allows you to re-configure an IIS web site with
' user-defined settings at the click of a button.
'
' Developed by Mark Gilbert (http://www.mark-gilbert.com).
'
' This software is license under the Microsoft Public License (Ms-PL).  For full
' license details, please see 
' http://www.microsoft.com/resources/sharedsource/licensingbasics/publiclicense.mspx.
' *****************************************************************************

Imports System.DirectoryServices
Imports System.IO

Public Class Main

    Private _WebSiteID As String

    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.

        ' In these first versions the only site that IIS Tweak can configure is http://localhost, which
        ' I believe is always defined as web site ID "1"
        Me._WebSiteID = "1"
        Me.LastSavedLabel.Text = ""
    End Sub


#Region " Properties "
    ''' <summary>
    ''' This exposes the web site's Home Directory property in the IIS metabase.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Property HomeDirectory() As String
        Get
            Dim MyLocalhost As DirectoryEntry
            Try
                MyLocalhost = New DirectoryEntry("IIS://localhost/w3svc/" & Me._WebSiteID & "/ROOT")
                Return MyLocalhost.Properties("Path").Item(0)
            Catch ex As Exception
                Throw ex
            End Try
        End Get
        Set(ByVal value As String)
            Dim MyLocalhost As DirectoryEntry

            Try
                MyLocalhost = New DirectoryEntry("IIS://localhost/w3svc/" & Me._WebSiteID & "/ROOT")
                MyLocalhost.Properties("Path").Item(0) = value
                MyLocalhost.CommitChanges()
            Catch ex As Exception
                Throw ex
            End Try
        End Set
    End Property


    ''' <summary>
    ''' This returns the absolute path to IIS Tweak's settings file.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property SettingsFile() As String
        Get
            Dim ExecutionPath As String
            ExecutionPath = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location)
            Return Path.Combine(ExecutionPath, "IISTweakSettings.xml")
        End Get
    End Property
#End Region


#Region " Private Methods "
    ''' <summary>
    ''' This updates the Version information in a label in the footer; it pulls this via reflection from the executing assembly
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LoadVersionInfo()
        Dim asnCurrent As System.Reflection.AssemblyName

        asnCurrent = System.Reflection.Assembly.GetExecutingAssembly.GetName
        Me.VersionLabel.Text = asnCurrent.Version.Major & "." & _
                                asnCurrent.Version.Minor.ToString.PadLeft(1, "0") & "." & _
                                asnCurrent.Version.Build.ToString.PadLeft(1, "0") & vbCrLf
    End Sub


    ''' <summary>
    ''' This updates the status box with the new message.
    ''' </summary>
    ''' <param name="NewMessage"></param>
    ''' <remarks></remarks>
    Private Sub UpdateStatus(ByVal NewMessage As String, ByVal IsError As Boolean)
        Dim ControlGray As System.Drawing.Color

        ControlGray = Color.FromArgb(255, 236, 233, 216)
        Me.StatusBox.Text = NewMessage.Trim


        If (IsError) Then
            Threading.Thread.Sleep(150)
            Me.Refresh()
            Me.StatusBox.BackColor = Color.Red
            Me.StatusBox.ForeColor = Color.White
            Me.Refresh()
            Threading.Thread.Sleep(250)
            Me.StatusBox.BackColor = ControlGray
            Me.StatusBox.ForeColor = Color.Black
            Me.Refresh()
            Threading.Thread.Sleep(250)
            Me.StatusBox.BackColor = Color.Red
            Me.StatusBox.ForeColor = Color.White
            Me.Refresh()
            Threading.Thread.Sleep(250)
            Me.StatusBox.BackColor = ControlGray
            Me.StatusBox.ForeColor = Color.Black
            Me.Refresh()
            Threading.Thread.Sleep(250)
            Me.StatusBox.BackColor = Color.Red
            Me.StatusBox.ForeColor = Color.White
            Me.Refresh()
        Else
            Me.StatusBox.BackColor = ControlGray
            Me.StatusBox.ForeColor = Color.Black
        End If
    End Sub


    ''' <summary>
    ''' As the name implies, this reads in the IIS Tweak settings from the file, and configures the utility.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LoadSettings()
        Dim SiteList As XDocument
        Dim CurrentSiteGroup As XElement
        Dim NewSiteGroupNode As TreeNode

        Me.MainNavigationTree.Items.Clear()

        If (File.Exists(Me.SettingsFile)) Then
            SiteList = XDocument.Load(Me.SettingsFile)

            Me.Height = SiteList...<height>.Value
            Me.Width = SiteList...<width>.Value

            Dim SiteGroups = From sg In SiteList...<sitegroup> Select sg
            For Each CurrentSiteGroup In SiteGroups
                NewSiteGroupNode = New TreeNode(CurrentSiteGroup.@name)

                For Each CurrentSiteNode In CurrentSiteGroup...<site>
                    NewSiteGroupNode.Nodes.Add(New SiteNode(CurrentSiteNode.@name, CurrentSiteNode.@path))
                Next

                Me.MainNavigationTree.Items.Add(NewSiteGroupNode)
            Next

            Me.MainNavigationTree.ExpandAll()
            Me.MainNavigationTree.ScrollToTop()
        End If
    End Sub


    ''' <summary>
    ''' This saves out the tree contents, as well as the utility's current width and height
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SaveSettings()
        Dim SettingsElements, SiteGroups, CurrentSiteGroup As XElement
        Dim SiteList As XDocument
        Dim CurrentTreeNode As TreeNode
        Dim CurrentSiteNode As SiteNode

        SiteGroups = <sitegroups></sitegroups>
        For Each CurrentTreeNode In Me.MainNavigationTree.Items
            CurrentSiteGroup = <sitegroup name=<%= CurrentTreeNode.Text %>></sitegroup>
            If (CurrentTreeNode.Nodes.Count > 0) Then
                For Each CurrentSiteNode In CurrentTreeNode.Nodes
                    CurrentSiteGroup.Add(<site name=<%= CurrentSiteNode.DisplayName %> path=<%= CurrentSiteNode.Path %>/>)
                Next
            End If

            SiteGroups.Add(CurrentSiteGroup)
        Next

        SettingsElements = <iis_tweak>
                               <appsettings>
                                   <width><%= Me.Width %></width>
                                   <height><%= Me.Height %></height>
                               </appsettings>
                               <%= SiteGroups %>
                           </iis_tweak>

        SiteList = New XDocument(New XDeclaration("1.0", "UTF-16", "yes"), SettingsElements)
        SiteList.Save(Me.SettingsFile)

        Me.LastSavedLabel.Text = "Last Saved " & Now.ToLongTimeString
    End Sub

#End Region



#Region " Event Handlers "
    Private Sub Main_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Me.LoadVersionInfo()
        Me.LoadSettings()
    End Sub


    Private Sub Main_Shown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Shown
        Try
            Me.UpdateStatus("Localhost home directory currently points to: " & Me.HomeDirectory, False)
        Catch ex As Exception
            Me.UpdateStatus("ERROR: The specified web site could not be found.  Please verify that it still exists, and try again.", True)
        End Try
    End Sub


    Private Sub CloseButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CloseButton.Click
        Me.Close()
    End Sub


    Private Sub MainNavigationTree_ItemExecuted(ByVal e As System.EventArgs) Handles MainNavigationTree.ItemExecuted
        If (IsNothing(Me.MainNavigationTree.SelectedItem)) Then Return

        Try
            Me.Cursor = Cursors.WaitCursor
            Me.HomeDirectory = Me.MainNavigationTree.SelectedItem.Path
            Me.UpdateStatus("Localhost home directory changed to: " & Me.HomeDirectory, False)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            Me.UpdateStatus("ERROR: The specified web site could not be found.  Please verify that it still exists, and try again.", True)
        Finally
            Me.Cursor = Cursors.Default
        End Try
    End Sub


    Private Sub MainNavigationTree_ItemsEdited(ByVal e As System.EventArgs) Handles MainNavigationTree.ItemsEdited
        Me.SaveSettings()
        Me.UpdateStatus("", False)
    End Sub


    Private Sub Main_ResizeEnd(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.ResizeEnd
        Me.SaveSettings()
        Me.UpdateStatus("", False)
    End Sub


    Private Sub MainNavigationTree_ModificationExceptionThrown(ByVal e As NavigationTree.ModificationExceptionEventArgs) Handles MainNavigationTree.ModificationExceptionThrown
        Me.UpdateStatus(e.ExceptionMessage, True)
    End Sub
#End Region
End Class
