﻿' *****************************************************************************
' IIS Tweak
' A simple utility that allows you to re-configure an IIS web site with
' user-defined settings at the click of a button.
'
' Developed by Mark Gilbert (http://www.mark-gilbert.com).
'
' This software is license under the Microsoft Public License (Ms-PL).  For full
' license details, please see 
' http://www.microsoft.com/resources/sharedsource/licensingbasics/publiclicense.mspx.
' *****************************************************************************
Public Class NavigationTree


#Region " Events and Exceptions "
    Public Event ItemExecuted(ByVal e As EventArgs)
    Public Event ItemsEdited(ByVal e As EventArgs)
    Public Event ModificationExceptionThrown(ByVal e As ModificationExceptionEventArgs)


    ''' <summary>
    ''' This class wraps the exception thrown into an EventArgs descendent that can be included
    ''' in a custom event. 
    ''' </summary>
    ''' <remarks>
    ''' This allows the code that includes the NavigationTree to simply catch the custom event
    ''' to handle exceptions that occur within.
    ''' </remarks>
    Public Class ModificationExceptionEventArgs
        Inherits EventArgs

        Private _InnerExceptionMessage As String

        Public Sub New(ByVal InnerExceptionMessage As String)
            Me.ExceptionMessage = InnerExceptionMessage
        End Sub

        Public Property ExceptionMessage()
            Get
                If (IsNothing(Me._InnerExceptionMessage)) Then Me._InnerExceptionMessage = ""
                If (Me._InnerExceptionMessage.Length = 0) Then Me._InnerExceptionMessage = "{no detail available}"
                Return Me._InnerExceptionMessage.Trim
            End Get
            Set(ByVal value)
                If (IsNothing(value)) Then value = ""
                Me._InnerExceptionMessage = value.trim
            End Set
        End Property
    End Class
#End Region



    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        Me.TitleLabel.Text = ""
        Me.MainTreeView.LabelEdit = True
    End Sub


#Region " Properties "
    ''' <summary>
    ''' This exposes the control's title, which sits above the treeview on the left side.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Title() As String
        Get
            Return Me.TitleLabel.Text.Trim
        End Get
        Set(ByVal value As String)
            If (IsNothing(value)) Then value = ""
            Me.TitleLabel.Text = value.Trim
        End Set
    End Property


    ''' <summary>
    ''' This exposes the Nodes collection, providing a slight abstraction for the underlying treeview structure.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Items() As TreeNodeCollection
        Get
            Return Me.MainTreeView.Nodes
        End Get
    End Property


    ''' <summary>
    ''' This returns the selected SiteNode node if the user has one selected.  If the user has a group selected, this returns Nothing.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property SelectedItem() As SiteNode
        Get
            If (IsNothing(Me.MainTreeView.SelectedNode)) Then Return Nothing
            If (Not Me.IsItemSelected) Then Return Nothing
            Return CType(Me.MainTreeView.SelectedNode, SiteNode)
        End Get
    End Property


    ''' <summary>
    ''' This returns True if the user has a SiteNode selected, and False otherwise.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property IsItemSelected() As Boolean
        Get
            If (IsNothing(Me.MainTreeView.SelectedNode)) Then Return False
            Return (Me.MainTreeView.SelectedNode.GetType.ToString = GetType(SiteNode).ToString)
        End Get
    End Property


    ''' <summary>
    ''' This returns True if the user has a group selected, and False otherwise.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property IsItemGroupSelected() As Boolean
        Get
            If (IsNothing(Me.MainTreeView.SelectedNode)) Then Return False
            Return (Me.MainTreeView.SelectedNode.GetType.ToString = GetType(TreeNode).ToString)
        End Get
    End Property
#End Region


#Region " Methods "
    ''' <summary>
    ''' This exposes the tree view's ExpandAll method.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub ExpandAll()
        Me.MainTreeView.ExpandAll()
    End Sub


    ''' <summary>
    ''' This forces the top-most node in the tree to be visible.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub ScrollToTop()
        If (IsNothing(Me.MainTreeView)) Then Return
        If (Me.MainTreeView.Nodes.Count = 0) Then Return
        Me.MainTreeView.TopNode = Me.MainTreeView.Nodes(0)
    End Sub


    ''' <summary>
    ''' This adds a new group to the end of the list.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub AddGroup()
        Me.MainTreeView.Nodes.Add("New Group")
        Me.MainTreeView.SelectedNode = Me.MainTreeView.Nodes(Me.MainTreeView.Nodes.Count - 1)
        Me.MainTreeView.SelectedNode.BeginEdit()
    End Sub


    ''' <summary>
    ''' This will prompt the user to confirm deletion of the selected item; if they confirm it will remove it, and raise the ItemsEdited event.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub RemoveNode()
        Dim CurrentNodeText As String

        CurrentNodeText = Me.MainTreeView.SelectedNode.Text
        If (Me.IsItemSelected) Then
            If (MsgBox("Do you really want to remove '" & CurrentNodeText & "'?", MsgBoxStyle.YesNo) = MsgBoxResult.No) Then Return
        Else
            If (MsgBox("This will remove the '" & CurrentNodeText & "' group, and all items contained in it.  Do you really want to do this?", MsgBoxStyle.YesNo) = MsgBoxResult.No) Then Return
        End If

        Me.MainTreeView.SelectedNode.Remove()
        RaiseEvent ItemsEdited(New EventArgs)
    End Sub


    ''' <summary>
    ''' This will add a new item, prompting the user for the path to the item
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub AddItem()
        Dim SelectDirectoryDialog As FolderBrowserDialog
        Dim NewSiteNode As SiteNode

        SelectDirectoryDialog = New FolderBrowserDialog
        SelectDirectoryDialog.Description = "Please select the web root for the site."


        If (SelectDirectoryDialog.ShowDialog = DialogResult.OK) Then
            NewSiteNode = New SiteNode("New Site", SelectDirectoryDialog.SelectedPath)

            If (Me.IsItemSelected) Then
                Me.MainTreeView.SelectedNode.Parent.Nodes.Add(NewSiteNode)
                Me.MainTreeView.SelectedNode = Me.MainTreeView.SelectedNode.Parent.LastNode
            Else
                Me.MainTreeView.SelectedNode.Nodes.Add(NewSiteNode)
                Me.MainTreeView.SelectedNode = Me.MainTreeView.SelectedNode.LastNode
            End If

            RaiseEvent ItemsEdited(New EventArgs)
        End If

        SelectDirectoryDialog.Dispose()
        SelectDirectoryDialog = Nothing
    End Sub


    ''' <summary>
    ''' This will begin editing of the selected tree node.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub RenameNode()
        If (Not IsNothing(Me.MainTreeView.SelectedNode)) Then Me.MainTreeView.SelectedNode.BeginEdit()
    End Sub


    ''' <summary>
    ''' This will refresh the UI controls when a new node in the tree has been selected.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub UpdateUIWhenNodeSelected()
        ' Tree buttons
        Me.NewFolderButton.Enabled = True
        Me.NewItemButton.Enabled = Me.IsItemSelected Or Me.IsItemGroupSelected
        Me.ExecuteButton.Enabled = Me.IsItemSelected
        Me.RenameButton.Enabled = Me.IsItemSelected Or Me.IsItemGroupSelected
        Me.DeleteButton.Enabled = Me.IsItemSelected Or Me.IsItemGroupSelected

        ' Context Menu buttons
        Me.AddGroupToolStripMenuItem.Enabled = True
        Me.AddItemToolStripMenuItem.Enabled = Me.IsItemSelected Or Me.IsItemGroupSelected
        Me.RunToolStripMenuItem.Enabled = Me.IsItemSelected
        Me.RenameToolStripMenuItem.Enabled = Me.IsItemSelected Or Me.IsItemGroupSelected
        Me.DeleteToolStripMenuItem.Enabled = Me.IsItemSelected Or Me.IsItemGroupSelected
    End Sub
#End Region


#Region " Event Handlers "
    Private Sub NavigationTree_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Me.UpdateUIWhenNodeSelected()
    End Sub


    Private Sub MainTreeView_AfterLabelEdit(ByVal sender As Object, ByVal e As System.Windows.Forms.NodeLabelEditEventArgs) Handles MainTreeView.AfterLabelEdit
        Try
            If (Me.IsItemSelected) Then CType(Me.MainTreeView.SelectedNode, SiteNode).DisplayName = e.Label
            e.CancelEdit = False
            RaiseEvent ItemsEdited(e)
        Catch ex As Exception
            RaiseEvent ModificationExceptionThrown(New ModificationExceptionEventArgs("An error occurred completing your last modification: " & ex.Message))
            e.CancelEdit = True
        End Try
    End Sub


    Private Sub MainTreeView_AfterSelect(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles MainTreeView.AfterSelect
        Me.UpdateUIWhenNodeSelected()
    End Sub


    Private Sub MainTreeView_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles MainTreeView.DoubleClick
        If (Me.IsItemSelected) Then RaiseEvent ItemExecuted(New EventArgs)
    End Sub


    Private Sub ExecuteButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExecuteButton.Click
        If (Me.IsItemSelected) Then RaiseEvent ItemExecuted(New EventArgs)
    End Sub


    Private Sub NewFolderButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NewFolderButton.Click
        Me.AddGroup()
    End Sub


    Private Sub DeleteButton_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles DeleteButton.Click
        Me.RemoveNode()
        Me.UpdateUIWhenNodeSelected()
    End Sub


    Private Sub NewItemButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NewItemButton.Click
        Me.AddItem()
    End Sub


    Private Sub RenameButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RenameButton.Click
        Me.RenameNode()
    End Sub


    Private Sub AddGroupToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AddGroupToolStripMenuItem.Click
        Me.AddGroup()
    End Sub


    Private Sub AddItemToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AddItemToolStripMenuItem.Click
        Me.AddItem()
    End Sub


    Private Sub RunToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RunToolStripMenuItem.Click
        RaiseEvent ItemExecuted(New EventArgs)
    End Sub


    Private Sub RenameToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RenameToolStripMenuItem.Click
        Me.RenameNode()
    End Sub


    Private Sub DeleteToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DeleteToolStripMenuItem.Click
        Me.RemoveNode()
        Me.UpdateUIWhenNodeSelected()
    End Sub


    ''' <summary>
    ''' This looks for and routes the F2 keystroke (rename), Delete keystroke (remove item), and Enter keystroke (execute item)
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainTreeView_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MainTreeView.KeyDown
        If (e.KeyCode = Keys.F2) Then
            Me.RenameNode()
        ElseIf (e.KeyCode = Keys.Delete) Then
            Me.RemoveNode()
            Me.UpdateUIWhenNodeSelected()
        ElseIf (e.KeyCode = Keys.Enter) Then
            If (Me.IsItemSelected) Then RaiseEvent ItemExecuted(New EventArgs)
        End If
    End Sub


    ''' <summary>
    ''' This forces the node that was clicked to be the selected node.  The two would be out of sync if the user
    ''' had one selected, and then right-clicked another.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainTreeView_NodeMouseClick(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeNodeMouseClickEventArgs) Handles MainTreeView.NodeMouseClick
        If (IsNothing(CType(sender, TreeView).SelectedNode)) Then CType(sender, TreeView).SelectedNode = e.Node
        If (Not CType(sender, TreeView).SelectedNode.Equals(e.Node)) Then Me.MainTreeView.SelectedNode = e.Node
    End Sub
#End Region
End Class
